#include <usb.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

/* the device's vendor and product id */
#define MY_VID 0x830
#define MY_PID 0x61

/* the device's endpoints */
#define EP_OUT 0x4
#define EP_IN 0x82

#define BUF_SIZE 16000
#define TIMEOUT  60000

char message_buffer[BUF_SIZE]; /* holds message data received or to be send */
usb_dev_handle *dev;  /* the usb device handle */


usb_dev_handle *open_dev(void)
{
    struct usb_bus *bus;
    struct usb_device *dev;

    for (bus = usb_get_busses(); bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            if (dev->descriptor.idVendor == MY_VID && dev->descriptor.idProduct == MY_PID) {
                printf("Palm found\n");
                return usb_open(dev);
            }
        }
    }

    return NULL;
}

int usb_connect()
{
    dev = NULL;

    usb_init();             /* initialize the library */
    usb_find_busses();      /* find all busses */
    usb_find_devices();     /* find all connected devices */

    if (!(dev = open_dev())) {
        //printf("ERROR: device not found!\n");
        return 0;
    }

    if (usb_set_configuration(dev, 1) < 0) {
        //printf("ERROR: setting config 1 failed\n");
        usb_close(dev);
        return 0;
    }

    if (usb_claim_interface(dev, 0) < 0) {
        //printf("ERROR: claiming interface 0 failed\n");
        usb_close(dev);
        return 0;
    }

    return 1;  /* connection established */
}

void usb_disconnect()
{
    usb_release_interface(dev, 0);
    usb_close(dev);
}

void usb_send(const char* pdata)
{  
   memset(message_buffer, '\0', BUF_SIZE);     
   memcpy(message_buffer, pdata, strlen(pdata));
   if (usb_bulk_write(dev, EP_OUT, message_buffer, BUF_SIZE, TIMEOUT) != BUF_SIZE) {
       /*printf("ERROR: USB send failed %s\n", usb_strerror());*/
   } else {
       printf("\nOK USB send:\n==============\n%s\n==============\n", pdata);
   }     
}

int usb_receive()
{
    int count = 0;
    memset(message_buffer, '\0', BUF_SIZE);
    count = usb_bulk_read(dev, EP_IN, message_buffer, BUF_SIZE, TIMEOUT);
    if (count <= 0) {
        /*printf("ERROR: USB receive failed %d %s\n", count, usb_strerror());*/
        return count;
    } else {
        printf("\nOK USB receive:\n==============\n%s\n==============\n", message_buffer);
        return count;
    }
}


int main(void)
{
    int count;
    int         running = 1;
    FILE*       pf;
    char        buffer[BUF_SIZE];
    size_t      size = 0;
    size_t      rcount = 0;
    int         c;

    while(running) {
             
        while(usb_connect() == 0); /* try connecting until a connection is established */       
        
        count = usb_receive();
        
        if (count > 0) {
            
            if (strcmp("BATTERY", message_buffer)==0) {
               pf = (FILE*)fopen("questions1.xml", "rb");
               if (pf == NULL) {
                      printf("Cannot open input file!\n");
                      continue;
               }
               fseek(pf, 0, SEEK_END);
               size=ftell(pf);
               fseek(pf, 0, SEEK_SET);

               if (size > BUF_SIZE) {
                  printf("File is larger than %d! Cannot continue...\n", BUF_SIZE);
               }

               memset(buffer, '\0', BUF_SIZE);
               
               printf("Sending %d bytes in %d buffer.\n", size+1, BUF_SIZE);
               
               rcount = fread(buffer, 1, size, pf);
                              
               if (rcount != size) {
                  printf("Partial read! Only %d bytes of %d read.\n", rcount, size);
               }
                  
               usb_send(buffer);
               
               fclose(pf);
               printf("Done.\n");
               running = 0;
            }            
            else if (strncmp("ANSWERS", message_buffer, strlen("ANSWERS"))==0) {
               usb_send("ANSWERS OK");
            }                   
        }
        
        usb_disconnect();
    }
    
    system("PAUSE");
    
    return 0;
}
